@@ -150,14 +150,18 @@ class Scheduler:
                 max_instances: int = -1,
                 replace_existing: bool = False,
                 trigger: Optional[triggers.Trigger] = None,
-                next_run_time: Optional[datetime] = None) -> Job:
+                next_run_time: Optional[datetime] = None,
+                active: bool = True,
+                callback: Optional[Callable] = None,
+                error_callback: Optional[Callable] = None) -> Job:
+        if next_run_time is not None and not active:
+            raise ValueError('Job can\'t be inactive and have a non-null next_run_time!')
+
         if identifier is None:
             identifier = uuid.uuid4()
 
-        now_dt = datetime.now(tz=self.tz)
-
         if trigger is None:
-            trigger = triggers.DateTrigger(now_dt)
+            trigger = triggers.DateTrigger(datetime.now(tz=self.tz))
 
         trigger.tz = self.tz
 
@@ -170,7 +174,10 @@ class Scheduler:
                   next_run_time=next_run_time,
                   description=description,
                   coalesce=coalesce,
-                  max_instances=max_instances)
+                  max_instances=max_instances,
+                  active=active,
+                  callback=callback,
+                  error_callback=error_callback)
 
         with self._jobstore_lock:
             self._jobstore.add_job(job, replace_existing)
@@ -178,18 +185,26 @@ class Scheduler:
         logger.info(f'Added job {identifier}')
         self._dispatch_event(events.JobEvent(events.EVENT_JOB_ADDED, identifier))
 
-        if self.state == State.RUNNING:
+        if self.state == State.RUNNING and active:
             self.wakeup()
 
         return job
 
-    def modify_job(self, job_id: str, **changes):
+    def modify_job(self, job_id: str, **changes) -> Job:
         with self._jobstore_lock:
             job = self.get_job(job_id)
+
+            new_next_run_time = changes['next_run_time'] if 'next_run_time' in changes else job.next_run_time
+            new_active = changes['active'] if 'active' in changes else job.active
+
+            if new_next_run_time is not None and not new_active:
+                raise ValueError('Job can\'t be inactive and have a non-null next_run_time!')
+
             job._modify(**changes)
 
             if job.trigger is None:
                 job.trigger = triggers.DateTrigger(datetime.now(tz=self.tz))
+
             job.trigger.tz = self.tz
 
             self._jobstore.update_job(job)
@@ -201,7 +216,7 @@ class Scheduler:
 
         return job
 
-    def get_job(self, job_id: str):
+    def get_job(self, job_id: str) -> Job:
         with self._jobstore_lock:
             return self._jobstore.get_job(job_id)
 
@@ -218,18 +233,26 @@ class Scheduler:
 
     def terminate_job(self, instance_id: int):
         with self._executor_lock:
-            self._executor.terminate_job(instance_id)
+            job_id = self._executor.terminate_job(instance_id)
+            logger.info(f'Terminated job {job_id}')
+            self._dispatch_event(events.JobEvent(events.EVENT_JOB_TERMINATED, job_id))
 
     def trigger_job(self, job_id: str) -> int:
         job = self.get_job(job_id)
         run_time = datetime.now(tz=self.tz)
-        instance_id = self._executor.submit_job(job, run_time)
+
+        try:
+            instance_id = self._executor.submit_job(job, run_time)
+        except executor.MaxJobInstancesReachedException:
+            self._dispatch_event(events.JobEvent(events.EVENT_JOB_MAX_INSTANCES, job_id))
+            raise
+
         return instance_id
 
     def get_job_instance_ids(self, job_id: str) -> List[int]:
         with self._jobstore_lock:
             if not self._jobstore.contains_job(job_id):
-                raise jobstores.JobDoesNotExistException
+                raise jobstores.JobDoesNotExistException(job_id)
 
         with self._executor_lock:
             return self._executor.get_job_instance_ids(job_id)
